@@ -72,6 +72,7 @@ from .entities import (
     VariablesEntry,
     VariablesImport,
 )
+from .errors import DIAGNOSTICS_SOURCE_NAME, Error
 from .imports_manager import ImportsManager
 from .library_doc import (
     BUILTIN_LIBRARY_NAME,
@@ -82,8 +83,6 @@ from .library_doc import (
     LibraryDoc,
 )
 
-DIAGNOSTICS_SOURCE_NAME = "robotcode.namespace"
-
 EXTRACT_COMMENT_PATTERN = re.compile(r".*(?:^ *|\t+| {2,})#(?P<comment>.*)$")
 ROBOTCODE_PATTERN = re.compile(r"(?P<marker>\brobotcode\b)\s*:\s*(?P<rule>\b\w+\b)")
 
@@ -247,7 +246,6 @@ class BlockVariableVisitor(Visitor):
         args: List[str] = []
         n = cast(Arguments, node)
         arguments = n.get_tokens(RobotToken.ARGUMENT)
-        argument_definitions = []
 
         for argument_token in (cast(RobotToken, e) for e in arguments):
             try:
@@ -275,14 +273,10 @@ class BlockVariableVisitor(Visitor):
                             keyword_doc=self.current_kw_doc,
                         )
                         self._results[argument.value] = arg_def
-                        argument_definitions.append(arg_def)
 
             except VariableError:
                 pass
 
-        if self.current_kw_doc is not None:
-            self.current_kw_doc.argument_definitions = argument_definitions
-
     def visit_ExceptHeader(self, node: ast.AST) -> None:  # noqa: N802
         from robot.errors import VariableError
         from robot.parsing.lexer.tokens import Token as RobotToken
@@ -564,6 +558,7 @@ class Namespace:
         self._diagnostics: List[Diagnostic] = []
         self._keyword_references: Dict[KeywordDoc, Set[Location]] = {}
         self._variable_references: Dict[VariableDefinition, Set[Location]] = {}
+        self._local_variable_assignments: Dict[VariableDefinition, Set[Range]] = {}
         self._namespace_references: Dict[LibraryEntry, Set[Location]] = {}
 
         self._imported_keywords: Optional[List[KeywordDoc]] = None
@@ -717,6 +712,13 @@ class Namespace:
 
         return self._variable_references
 
+    async def get_local_variable_assignments(self) -> Dict[VariableDefinition, Set[Range]]:
+        await self.ensure_initialized()
+
+        await self._analyze()
+
+        return self._local_variable_assignments
+
     async def get_namespace_references(self) -> Dict[LibraryEntry, Set[Location]]:
         await self.ensure_initialized()
 
@@ -1048,7 +1050,7 @@ class Namespace:
                             message=f"Imported library '{value.name}' contains no keywords.",
                             severity=DiagnosticSeverity.WARNING,
                             source=DIAGNOSTICS_SOURCE_NAME,
-                            code="LibraryContainsNoKeywords",
+                            code=Error.LIBRARY_CONTAINS_NO_KEYWORDS,
                         )
                 elif isinstance(value, ResourceImport):
                     if value.name is None:
@@ -1071,7 +1073,7 @@ class Namespace:
                                 ]
                                 if value.source
                                 else None,
-                                code="PossibleCircularImport",
+                                code=Error.POSSIBLE_CIRCULAR_IMPORT,
                             )
                     else:
                         result = await self._get_resource_entry(
@@ -1096,7 +1098,7 @@ class Namespace:
                                 message=f"Imported resource file '{value.name}' is empty.",
                                 severity=DiagnosticSeverity.WARNING,
                                 source=DIAGNOSTICS_SOURCE_NAME,
-                                code="ResourceEmpty",
+                                code=Error.RESOURCE_EMPTY,
                             )
 
                 elif isinstance(value, VariablesImport):
@@ -1151,7 +1153,7 @@ class Namespace:
                                     for err in result.library_doc.errors
                                     if err.source is not None
                                 ],
-                                code="ImportContainsErrors",
+                                code=Error.IMPORT_CONTAINS_ERRORS,
                             )
                         for err in filter(
                             lambda e: e.source is None or not Path(e.source).is_absolute(), result.library_doc.errors
@@ -1236,7 +1238,7 @@ class Namespace:
                                         message="Recursive resource import.",
                                         severity=DiagnosticSeverity.INFORMATION,
                                         source=DIAGNOSTICS_SOURCE_NAME,
-                                        code="RecursiveImport",
+                                        code=Error.RECURSIVE_IMPORT,
                                     )
                                 elif (
                                     already_imported_resources is not None
@@ -1258,7 +1260,7 @@ class Namespace:
                                         ]
                                         if already_imported_resources.import_source
                                         else None,
-                                        code="ResourceAlreadyImported",
+                                        code=Error.RESOURCE_ALREADY_IMPORTED,
                                     )
 
                     elif isinstance(entry, VariablesEntry):
@@ -1290,7 +1292,7 @@ class Namespace:
                                 ]
                                 if already_imported_variables[0].import_source
                                 else None,
-                                code="VariablesAlreadyImported",
+                                code=Error.VARIABLES_ALREADY_IMPORTED,
                             )
 
                         if (entry.alias or entry.name or entry.import_name) not in self._variables:
@@ -1315,7 +1317,7 @@ class Namespace:
                                 ]
                                 if entry.import_source
                                 else None,
-                                code="LibraryOverridesBuiltIn",
+                                code=Error.LIBRARY_OVERRIDES_BUILTIN,
                             )
                             continue
 
@@ -1343,7 +1345,7 @@ class Namespace:
                                 ]
                                 if already_imported_library[0].import_source
                                 else None,
-                                code="LibraryAlreadyImported",
+                                code=Error.LIBRARY_ALREADY_IMPORTED,
                             )
 
                         if (entry.alias or entry.name or entry.import_name) not in self._libraries:
@@ -1583,6 +1585,7 @@ class Namespace:
                     self._diagnostics += result.diagnostics
                     self._keyword_references = result.keyword_references
                     self._variable_references = result.variable_references
+                    self._local_variable_assignments = result.local_variable_assignments
                     self._namespace_references = result.namespace_references
 
                     lib_doc = await self.get_library_doc()
@@ -1691,8 +1694,10 @@ class KeywordFinder:
         self.self_library_doc = library_doc
 
         self.diagnostics: List[DiagnosticsEntry] = []
-        self.multiple_keywords_result: Optional[Sequence[Tuple[Optional[LibraryEntry], KeywordDoc]]] = None
-        self._cache: Dict[Tuple[Optional[str], bool], Tuple[Optional[KeywordDoc], List[DiagnosticsEntry]]] = {}
+        self.multiple_keywords_result: Optional[List[KeywordDoc]] = None
+        self._cache: Dict[
+            Tuple[Optional[str], bool], Tuple[Optional[KeywordDoc], List[DiagnosticsEntry], Optional[List[KeywordDoc]]]
+        ] = {}
         self.handle_bdd_style = True
         self._all_keywords: Optional[List[LibraryEntry]] = None
         self._resource_keywords: Optional[List[ResourceEntry]] = None
@@ -1714,6 +1719,7 @@ class KeywordFinder:
 
             if cached is not None:
                 self.diagnostics = cached[1]
+                self.multiple_keywords_result = cached[2]
                 return cached[0]
 
             try:
@@ -1723,17 +1729,20 @@ class KeywordFinder:
                         DiagnosticsEntry(
                             f"No keyword with name '{name}' found.",
                             DiagnosticSeverity.ERROR,
-                            "KeywordNotFoundError",
+                            Error.KEYWORD_NOT_FOUND,
                         )
                     )
             except KeywordError as e:
+                if e.multiple_keywords:
+                    self._add_to_multiple_keywords_result(e.multiple_keywords)
+
                 if raise_keyword_error:
                     raise
 
                 result = None
-                self.diagnostics.append(DiagnosticsEntry(str(e), DiagnosticSeverity.ERROR, "KeywordError"))
+                self.diagnostics.append(DiagnosticsEntry(str(e), DiagnosticSeverity.ERROR, Error.KEYWORD_ERROR))
 
-            self._cache[(name, self.handle_bdd_style)] = (result, self.diagnostics)
+            self._cache[(name, self.handle_bdd_style)] = (result, self.diagnostics, self.multiple_keywords_result)
 
             return result
         except CancelSearchError:
@@ -1742,12 +1751,12 @@ class KeywordFinder:
     def _find_keyword(self, name: Optional[str]) -> Optional[KeywordDoc]:
         if not name:
             self.diagnostics.append(
-                DiagnosticsEntry("Keyword name cannot be empty.", DiagnosticSeverity.ERROR, "KeywordError")
+                DiagnosticsEntry("Keyword name cannot be empty.", DiagnosticSeverity.ERROR, Error.KEYWORD_ERROR)
             )
             raise CancelSearchError
         if not isinstance(name, str):
             self.diagnostics.append(  # type: ignore
-                DiagnosticsEntry("Keyword name must be a string.", DiagnosticSeverity.ERROR, "KeywordError")
+                DiagnosticsEntry("Keyword name must be a string.", DiagnosticSeverity.ERROR, Error.KEYWORD_ERROR)
             )
             raise CancelSearchError
 
@@ -1775,7 +1784,7 @@ class KeywordFinder:
                         DiagnosticsEntry(
                             self._create_multiple_keywords_found_message(name, found, implicit=False),
                             DiagnosticSeverity.ERROR,
-                            "KeywordError",
+                            Error.KEYWORD_ERROR,
                         )
                     )
                     raise CancelSearchError
@@ -1793,7 +1802,7 @@ class KeywordFinder:
                 DiagnosticsEntry(
                     str(e),
                     DiagnosticSeverity.ERROR,
-                    "KeywordError",
+                    Error.KEYWORD_ERROR,
                 )
             )
             raise CancelSearchError from e
@@ -1816,7 +1825,7 @@ class KeywordFinder:
                 DiagnosticsEntry(
                     self._create_multiple_keywords_found_message(name, found, implicit=False),
                     DiagnosticSeverity.ERROR,
-                    "KeywordError",
+                    Error.KEYWORD_ERROR,
                 )
             )
             raise CancelSearchError
@@ -1842,10 +1851,16 @@ class KeywordFinder:
                     result.append((v, kw))
         return result
 
+    def _add_to_multiple_keywords_result(self, kw: Iterable[KeywordDoc]) -> None:
+        if self.multiple_keywords_result is None:
+            self.multiple_keywords_result = list(kw)
+        else:
+            self.multiple_keywords_result.extend(kw)
+
     def _create_multiple_keywords_found_message(
         self, name: str, found: Sequence[Tuple[Optional[LibraryEntry], KeywordDoc]], implicit: bool = True
     ) -> str:
-        self.multiple_keywords_result = found
+        self._add_to_multiple_keywords_result([k for _, k in found])
 
         if any(e[1].is_embedded for e in found):
             error = f"Multiple keywords matching name '{name}' found"
@@ -1948,7 +1963,7 @@ class KeywordFinder:
             DiagnosticsEntry(
                 self._create_multiple_keywords_found_message(name, found),
                 DiagnosticSeverity.ERROR,
-                "KeywordError",
+                Error.KEYWORD_ERROR,
             )
         )
         raise CancelSearchError
@@ -2002,7 +2017,7 @@ class KeywordFinder:
             DiagnosticsEntry(
                 self._create_multiple_keywords_found_message(name, found),
                 DiagnosticSeverity.ERROR,
-                "KeywordError",
+                Error.KEYWORD_ERROR,
             )
         )
         raise CancelSearchError
@@ -2024,7 +2039,7 @@ class KeywordFinder:
             DiagnosticsEntry(
                 self._create_custom_and_standard_keyword_conflict_warning_message(custom, standard),
                 DiagnosticSeverity.WARNING,
-                "KeywordError",
+                Error.KEYWORD_ERROR,
             )
         )
 
